package org.richfaces.component;

import org.richfaces.component.events.ScheduleViewChangedEvent;
import org.richfaces.component.events.ScheduleLoadEvent;
import org.richfaces.component.events.ScheduleItemResizedEvent;
import org.richfaces.component.events.ScheduleItemSelectedEvent;
import org.ajax4jsf.context.AjaxContext;
import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;

import javax.el.ELContext;
import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.FacesEvent;
import javax.faces.model.*;
import javax.servlet.jsp.jstl.sql.Result;
import java.io.IOException;
import java.sql.ResultSet;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import org.richfaces.component.events.ScheduleItemMovedEvent;
import org.richfaces.component.model.DateRange;

/**
 * JSF component class
 */
public abstract class UISchedule extends UIComponentBase {

    public static final String COMPONENT_TYPE = "org.richfaces.Schedule";
    public static final String COMPONENT_FAMILY = "org.richfaces.Schedule";
    public static final String VIEW_MONTH = "month";
    public static final String VIEW_BASIC_WEEK = "basicWeek";
    public static final String VIEW_AGENDA_WEEK = "agendaWeek";
    public static final String VIEW_BASIC_DAY = "basicDay";
    public static final String VIEW_AGENDA_DAY = "agendaDay";
    public static final String WEEK_MODE_FIXED = "fixed";
    private DataModel model;

    public abstract Object getValue();

    public abstract void setValue(Object value);

    public abstract String getVar();

    public abstract void setVar(String value);

    public abstract Date getDate();

    public abstract void setDate(Date date);

    public abstract String getMode();

    public abstract void setMode(String mode);

    public abstract String getView();

    public abstract void setView(String defaultView);

    public abstract String getHeaderLeft();

    public abstract void setHeaderLeft(String headerLeft);

    public abstract String getHeaderCenter();

    public abstract void setHeaderCenter(String headerCenter);

    public abstract String getHeaderRight();

    public abstract void setHeaderRight(String headerRight);

    public abstract Integer getFirstDay();

    public abstract void setFirstDay(Integer firstDay);

    public abstract Boolean getIsRTL();

    public abstract void setIsRTL(Boolean isrtl);

    public abstract Boolean getShowWeekends();

    public abstract void setShowWeekends(Boolean showWeekends);

    public abstract String getWeekMode();

    public abstract void setWeekMode(String weekMode);

    public abstract Integer getHeight();

    public abstract void setHeight(Integer height);

    public abstract Integer getContentHeight();

    public abstract void setContentHeight(Integer contentHeight);

    public abstract Double getAspectRatio();

    public abstract void setAspectRatio(Double aspectRatio);

    public abstract Boolean getAllDayByDefault();

    public abstract void setAllDayByDefault(Boolean allDayByDefault);

    public abstract Boolean getAllDaySlot();

    public abstract void setAllDaySlot(Boolean allDaySlot);

    public abstract String getAllDayText();

    public abstract void setAllDayText(String allDayText);

    public abstract String getAxisFormat();

    public abstract void setAxisFormat(String axisFormat);

    public abstract Integer getSlotMinutes();

    public abstract void setSlotMinutes(Integer slotMinutes);

    public abstract Integer getDefaultEventMinutes();

    public abstract void setDefaultEventMinutes(Integer defaultEventMinutes);

    public abstract Integer getFirstHour();

    public abstract void setFirstHour(Integer firstHour);

    public abstract String getMinTime();

    public abstract void setMinTime(String minTime);

    public abstract String getMaxTime();

    public abstract void setMaxTime(String maxTime);

    public abstract Boolean getEditable();

    public abstract void setEditable(Boolean editable);

    public abstract Boolean getDisableDragging();

    public abstract void setDisableDragging(Boolean disableDragging);

    public abstract Boolean getDisableResizing();

    public abstract void setDisableResizing(Boolean disableResizing);

    public abstract Integer getDragRevertDuration();

    public abstract void setDragRevertDuration(Integer dragRevertDuration);

    public abstract Double getDragOpacity();

    public abstract void setDragOpacity(Double dragOpacity);

    // TODO do we use MethodBinding or MethodExpression?
    public abstract MethodBinding getItemMovedListener();

    public abstract void setItemMovedListener(MethodBinding listener);

    public abstract MethodBinding getItemSelectedListener();

    public abstract void setItemSelectedListener(MethodBinding listener);

    public abstract MethodBinding getItemResizedListener();

    public abstract void setItemResizedListener(MethodBinding listener);

    public abstract MethodBinding getViewChangedListener();

    public abstract void setViewChangedListener(MethodBinding listener);

//    TODO daterangeChangedEvent
    //TODO problem when h:messages are in a4j:outputPanel and it is ajaxRendered, cause when events are loading they erase messages (so messages displayed on standard form submit will disapear at once)
    @Override
    public void broadcast(FacesEvent event) throws AbortProcessingException {
        if (event instanceof ScheduleLoadEvent) {
            ScheduleLoadEvent calendarAjaxEvent = (ScheduleLoadEvent) event;
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            try {
//                TODO How about merging this data with data requested by user "<rich:schedule data="tralalala"/>
                ajaxContext.setResponseData(getCalendarData(calendarAjaxEvent.getStartDate(), calendarAjaxEvent.getEndDate()));
            } catch (IOException ex) {
                throw new AbortProcessingException(ex);
            }

        } else if (event instanceof ScheduleItemMovedEvent) {
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            MethodBinding expression = getItemMovedListener();
            boolean allow = true;
            if (expression != null) {
                Object result = expression.invoke(facesContext, new Object[]{event});
                allow = ((Boolean) result);
            }
            ajaxContext.setResponseData(allow);
        } else if (event instanceof ScheduleItemResizedEvent) {
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            MethodBinding expression = getItemResizedListener();
            boolean allow = true;
            if (expression != null) {
                Object result = expression.invoke(facesContext, new Object[]{event});
                allow = ((Boolean) result);
            }
            ajaxContext.setResponseData(allow);
        } else if (event instanceof ScheduleItemSelectedEvent) {
            FacesContext facesContext = getFacesContext();
            MethodBinding expression = getItemSelectedListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
        } else if (event instanceof ScheduleViewChangedEvent) {
            FacesContext facesContext = getFacesContext();
            MethodBinding expression = getViewChangedListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
        }
    }

    /**
     * Range is [startDate;endDate), which means that start date is inclusive
     * and end date is exclusive.
     *
     * @param startDate
     * @param endDate
     * @return
     * @throws IOException
     */
    public List<Map<String, Object>> getCalendarData(Date startDate, Date endDate) throws IOException {
        /**
         * Following code served for generation of test data
         */
        /*
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        calendar.add(Calendar.DATE, 9);
        setDate(calendar.getTime());
        final Date firstDisplayedDay = getFirstDisplayedDay2(this);
        Calendar firstDayCalendar = Calendar.getInstance();
        firstDayCalendar.setTime(firstDisplayedDay);
        Calendar startDateCalendar = Calendar.getInstance();
        startDateCalendar.setTime(startDate);
        Calendar endDateCalendar = Calendar.getInstance();
        endDateCalendar.setTime(endDate);
        Calendar lastDayCalendar = Calendar.getInstance();
        lastDayCalendar.setTime(getLastDisplayedDate(this));
        System.out.println("" + getFirstDay() + "," + (isShowWeekends(this) ? 1 : 0) +
                "," + calendar.get(Calendar.YEAR) + "," + calendar.get(Calendar.MONTH) + "," + calendar.get(Calendar.DATE)
                + "," + startDateCalendar.get(Calendar.YEAR) + "," + startDateCalendar.get(Calendar.MONTH) + "," + startDateCalendar.get(Calendar.DATE)
//                + "," + firstDayCalendar.get(Calendar.YEAR) + "," + firstDayCalendar.get(Calendar.MONTH) + "," + firstDayCalendar.get(Calendar.DATE)
                + "," + endDateCalendar.get(Calendar.YEAR) + "," + endDateCalendar.get(Calendar.MONTH) + "," + endDateCalendar.get(Calendar.DATE)
//                + "," + lastDayCalendar.get(Calendar.YEAR) + "," + lastDayCalendar.get(Calendar.MONTH) + "," + lastDayCalendar.get(Calendar.DATE)
        );
        */
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
        format.setLenient(false);
        DataModel dataModel = getDataModel();
        if (dataModel instanceof ExtendedDataModel) {
            DataVisitor visitor = new DataVisitor() {
                //TODO Is this fine? or should we stack rowKeys and not use dataModel later on. I'don't know business rules of extendedDataModel, just used it to did once pagination with underlying EntityQuery from Seam

                public void process(FacesContext context, Object rowKey, Object argument) throws IOException {
                }
            };
            /**
             * TODO Lets skip this date range and just fire appropriate events. Backing bean should provide datamodel aware of new range.
             */
            ((ExtendedDataModel) dataModel).walk(getFacesContext(), visitor, new DateRange(startDate, endDate), null);
        }
        ELContext elContext = getFacesContext().getELContext();
        ValueExpression valueExpression = getFacesContext().getApplication().getExpressionFactory().createValueExpression(getFacesContext().getELContext(), "#{" + getVar() + "}", Object.class);
        List<Map<String, Object>> data = new ArrayList<Map<String, Object>>();
        for (int i = 0; i < dataModel.getRowCount(); i++) {
            dataModel.setRowIndex(i);
            valueExpression.setValue(elContext, dataModel.getRowData());
            Map<String, Object> firstDataElement = new HashMap<String, Object>();
// TODO shouldn't we check earlier if there is exactly one renderable UIScheduleItem child?           
            for (UIComponent child : getChildren()) {
                if (child instanceof UIScheduleItem) {
                    UIScheduleItem item = (UIScheduleItem) child;
                    if (!item.isRendered()) {
                        continue;
                    }
                    firstDataElement.put("id", item.getEventId());
                    firstDataElement.put("title", item.getTitle());
                    if (item.getAllDay() != null) {
                        firstDataElement.put("allDay", item.getAllDay());
                    }
                    firstDataElement.put("start", format.format(item.getStartDate()));
                    if (item.getEndDate() != null) {
                        firstDataElement.put("end", format.format(item.getEndDate()));
                    }
                    if (item.getUrl() != null) {
                        firstDataElement.put("url", item.getUrl());
                    }
                    if (item.getStyleClass() != null) {
                        firstDataElement.put("className", item.getStyleClass());
                    }
                    if (item.getEditable() != null) {
                        firstDataElement.put("editable", item.getEditable());
                    }
                    data.add(firstDataElement);
                }
            }
        }
        valueExpression.setValue(elContext, null);
        return data;
    }

    public static Date getFirstDisplayedDay(UISchedule schedule) {
        Date date = schedule.getDate();
        Calendar calendar = Calendar.getInstance();
        if (date != null) {
            calendar.setTime(date);
        }
        calendar.set(Calendar.DATE, 1);
        boolean showWeekends = isShowWeekends(schedule);

        if (!showWeekends) {
            int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
            if (dayOfWeek == Calendar.SUNDAY) {
                calendar.add(Calendar.DATE, 1);
            } else if (dayOfWeek == Calendar.SATURDAY) {
                calendar.add(Calendar.DATE, 2);
            }
        }
        int dayOfWeekOfFirstDisplayedDayInMonth = calendar.get(Calendar.DAY_OF_WEEK);
        final Integer scheduleFirstDay = schedule.getFirstDay();
        int daysToSubtract = scheduleFirstDay - calendar.get(Calendar.DAY_OF_WEEK);
        if (daysToSubtract > 0) {
            daysToSubtract -= showWeekends || scheduleFirstDay == Calendar.SUNDAY || scheduleFirstDay == Calendar.SATURDAY ? 7 : 5;
        }
        calendar.add(Calendar.DAY_OF_YEAR, daysToSubtract);
        if (!showWeekends) {
            int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
            /**
             * If firstDay is saturday and showWeekends is false, then saturday is shown, but sunday is not.
             * If firstDay is sunday and showWeekends is false, then sunday is NOT shown and neither is saturday
             * (strange, but thats the way the fullcalendar widget works).
             */
            if ((dayOfWeek == Calendar.SUNDAY && scheduleFirstDay != Calendar.SUNDAY) || (dayOfWeek > Calendar.SUNDAY && dayOfWeek < Calendar.SATURDAY && dayOfWeek > dayOfWeekOfFirstDisplayedDayInMonth) || (dayOfWeek == Calendar.SATURDAY && Calendar.SATURDAY != scheduleFirstDay)) {
                calendar.add(Calendar.DAY_OF_YEAR, -2);
            } else if (dayOfWeek == Calendar.SUNDAY) {
                calendar.add(Calendar.DAY_OF_YEAR, 1);
            }
        }
        return calendar.getTime();
    }

    public static Date getFirstDisplayedDay2(UISchedule schedule) {
        Calendar calendar = Calendar.getInstance();
        Date date = schedule.getDate();
        if (date != null) {
            calendar.setTime(date);
        }
        int firstDayOfWeek = getFirstDay(schedule);
        calendar.setFirstDayOfWeek(firstDayOfWeek);
        String view = getView(schedule);
        if (VIEW_MONTH.equals(view)) {
            boolean showWeekends = isShowWeekends(schedule);
            calendar.set(Calendar.DATE, 1);
            if (!showWeekends) {
                int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
                if (dayOfWeek == Calendar.SUNDAY) {
                    calendar.add(Calendar.DATE, 1);
                } else if (dayOfWeek == Calendar.SATURDAY) {
                    calendar.add(Calendar.DATE, 2);
                }
            }
            if (!showWeekends && firstDayOfWeek == Calendar.SUNDAY) {
                firstDayOfWeek++;
            }
            /**
             * Following 1 line is a fix to what i believe is a bug of java.util.Calendar
             */
            calendar.get(Calendar.DAY_OF_WEEK);
            calendar.set(Calendar.DAY_OF_WEEK, firstDayOfWeek);
            return calendar.getTime();
        } else if (VIEW_AGENDA_WEEK.equals(view) || VIEW_BASIC_WEEK.equals(view)) {
            calendar.set(Calendar.DAY_OF_WEEK, firstDayOfWeek);
            return calendar.getTime();
        } else if (VIEW_AGENDA_DAY.equals(view) || VIEW_BASIC_DAY.equals(view)) {
            return calendar.getTime();
        } else {
            throw new IllegalStateException("Invalid view attribute: " + view);
        }
    }

    public static Date getLastDisplayedDate(UISchedule schedule) {
        Calendar calendar = Calendar.getInstance();
        if (WEEK_MODE_FIXED.equals(getWeekMode(schedule))) {
            calendar.setTime(getFirstDisplayedDay(schedule));
            calendar.add(Calendar.DAY_OF_YEAR, 7 * 6);
        } else {
            Date date = schedule.getDate();
            if (date != null) {
                calendar.setTime(date);
            }
            int firstDayOfWeek = getFirstDay(schedule);
            if (!isShowWeekends(schedule) && firstDayOfWeek == Calendar.SUNDAY) {
                firstDayOfWeek++;
            }
            calendar.setFirstDayOfWeek(firstDayOfWeek);
            calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
            int dayOfWeek = firstDayOfWeek + 6;
            if (dayOfWeek > Calendar.SATURDAY) {
                dayOfWeek -= 7;
            }
            /**
             * Following 1 line is a fix to what i believe is a bug of java.util.Calendar
             */
            calendar.get(Calendar.DAY_OF_WEEK);
            calendar.set(Calendar.DAY_OF_WEEK, dayOfWeek);
            calendar.add(Calendar.DATE, 1);
        }
        return calendar.getTime();
    }

    /**
     * Here go static methods for getting value or default value of attributes.
     */

    /**
     * 
     * @param schedule
     * @return
     */
    public static boolean isShowWeekends(UISchedule schedule) {
        Boolean showWeekends = schedule.getShowWeekends();
        return showWeekends == null || schedule.getShowWeekends();
    }

    public static int getFirstDay(UISchedule schedule) {
        Integer firstDay = schedule.getFirstDay();
        if (firstDay == null) {
            firstDay = Calendar.SUNDAY;
        }
        return firstDay;
    }

    public static String getWeekMode(UISchedule schedule) {
        String weekMode = schedule.getWeekMode();
        if (weekMode == null) {
            weekMode = WEEK_MODE_FIXED;
        }
        return weekMode;
    }

    public static String getView(UISchedule schedule) {
        String view = schedule.getView();
        if (view == null) {
            view = VIEW_MONTH;
        }
        return view;
    }

    protected DataModel getDataModel() {
//TODO check this method, it was just a quick c'n'p
        // Return any previously cached DataModel instance
        if (this.model != null) {
            return (model);
        }

        // Synthesize a DataModel around our current value if possible
        Object current = getValue();
        if (current == null) {
            setDataModel(new ListDataModel(Collections.EMPTY_LIST));
        } else if (current instanceof DataModel) {
            setDataModel((DataModel) current);
        } else if (current instanceof List) {
            setDataModel(new ListDataModel((List) current));
        } else if (Object[].class.isAssignableFrom(current.getClass())) {
            setDataModel(new ArrayDataModel((Object[]) current));
        } else if (current instanceof ResultSet) {
            setDataModel(new ResultSetDataModel((ResultSet) current));
        } else if (current instanceof Result) {
            setDataModel(new ResultDataModel((Result) current));
        } else {
            setDataModel(new ScalarDataModel(current));
        }
        return (model);

    }

    public void setDataModel(DataModel model) {
        this.model = model;
    }
}
