package org.richfaces.renderkit;

import org.richfaces.component.events.ScheduleViewChangedEvent;
import org.richfaces.component.events.ScheduleLoadEvent;
import org.richfaces.component.events.ScheduleItemResizedEvent;
import org.richfaces.component.events.ScheduleItemSelectedEvent;
import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.javascript.JSFunctionDefinition;
import org.ajax4jsf.javascript.JSObject;
import org.ajax4jsf.javascript.JSReference;
import org.ajax4jsf.renderkit.AjaxComponentRendererBase;
import org.ajax4jsf.renderkit.AjaxRendererUtils;
import org.richfaces.component.*;

import javax.faces.component.NamingContainer;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import org.richfaces.component.events.ScheduleItemMovedEvent;

public abstract class ScheduleRendererBase extends AjaxComponentRendererBase {

    public static final String ITEM_MOVED_EVENT = "itemMoved";
    public static final String ITEM_RESIZED_EVENT = "itemResized";
    public static final String ITEM_SELECTED_EVENT = "itemSelected";
    public static final String LOAD_ITEMS_EVENT = "loadItems";
    public static final String VIEW_CHANGED_EVENT = "viewChanged";
    public static final String AJAX_MODE = "ajax";
    public static final String SERVER_MODE = "server";
    public static final String CLIENT_MODE = "client";
    private static final String CALLBACK = "callback";
    private static final String END_DATE_PARAM = "endDate";
    private static final String START_DATE_PARAM = "startDate";
    private static final String EVENT_ID_PARAM = "eventId";
    private static final String DAY_DELTA_PARAM = "dayDelta";
    private static final String MINUTE_DELTA_PARAM = "minuteDelta";
    private static final String ALL_DAY_PARAM = "allDay";
    private static final String EVENT_TYPE_PARAM = "eventType";
    private static final String VIEW_PARAM = "view";

    protected Object createSubmitEventFunction(FacesContext context, UISchedule component) {
        JSFunction jsFunction = null;
        Map<String, Object> params = new HashMap<String, Object>();
        params.put(getFieldId(context, component, START_DATE_PARAM), new JSReference(START_DATE_PARAM));
        params.put(getFieldId(context, component, END_DATE_PARAM), new JSReference(END_DATE_PARAM));
        params.put(getFieldId(context, component, EVENT_ID_PARAM), new JSReference(EVENT_ID_PARAM));
        params.put(getFieldId(context, component, DAY_DELTA_PARAM), new JSReference(DAY_DELTA_PARAM));
        params.put(getFieldId(context, component, MINUTE_DELTA_PARAM), new JSReference(MINUTE_DELTA_PARAM));
        params.put(getFieldId(context, component, ALL_DAY_PARAM), new JSReference(ALL_DAY_PARAM));
        params.put(getFieldId(context, component, EVENT_TYPE_PARAM), new JSReference(EVENT_TYPE_PARAM));
        params.put(getFieldId(context, component, VIEW_PARAM), new JSReference(VIEW_PARAM));
        if (isAjaxMode(component)) {
            jsFunction = AjaxRendererUtils.buildAjaxFunction(component, context);
            Map<String, Object> eventOptions = AjaxRendererUtils.buildEventOptions(context, component, params);
            eventOptions.put("oncomplete", new JSReference(CALLBACK));
            jsFunction.addParameter(eventOptions);
        } else if (SERVER_MODE.equals(component.getMode())) {
            jsFunction = new JSFunction("Richfaces.jsFormSubmit",
                    component.getClientId(context),
                    getUtils().getNestingForm(context, component).getClientId(context),
                    null,
                    params);
        } else {
            return null;
        }
        return new JSFunctionDefinition("event", VIEW_PARAM, EVENT_TYPE_PARAM, EVENT_ID_PARAM, START_DATE_PARAM, END_DATE_PARAM, DAY_DELTA_PARAM, MINUTE_DELTA_PARAM, ALL_DAY_PARAM, CALLBACK).addToBody(jsFunction);
    }

    @Override
    public void decode(FacesContext context, UIComponent component) {
        if (!component.isRendered()) {
            return;
        }
//TODO what do we do with "view" property? Should we push it back via EL to backing bean like UIInput does, or just rely on ScheduleViewChangedEvent
//TODO view changed event
        Map<String, String> requestParameterMap = context.getExternalContext().getRequestParameterMap();
        if (requestParameterMap.get(component.getClientId(context)) != null) {
            String startDateParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, START_DATE_PARAM));
            String endDateParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, END_DATE_PARAM));
            String eventIdParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, EVENT_ID_PARAM));
            String dayDeltaParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, DAY_DELTA_PARAM));
            String minuteDeltaParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, MINUTE_DELTA_PARAM));
            String allDayParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, ALL_DAY_PARAM));
            String eventTypeParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, EVENT_TYPE_PARAM));
            String viewParam = requestParameterMap.get(getFieldId(context, (UISchedule) component, VIEW_PARAM));

            try {
                if (LOAD_ITEMS_EVENT.equals(eventTypeParam)) {
                    Date startDate = new Date(Long.parseLong(startDateParam) * 1000);
                    Date endDate = new Date(Long.parseLong(endDateParam) * 1000);
                    new ScheduleLoadEvent(component, startDate, endDate).queue();
                } else if (ITEM_MOVED_EVENT.equals(eventTypeParam)) {
                    int dayDelta = Integer.parseInt(dayDeltaParam);
                    int minuteDelta = Integer.parseInt(minuteDeltaParam);
                    boolean allDay = Boolean.parseBoolean(allDayParam);
                    new ScheduleItemMovedEvent(component, eventIdParam, dayDelta, minuteDelta, allDay).queue();
                } else if (ITEM_RESIZED_EVENT.equals(eventTypeParam)) {
                    int dayDelta = Integer.parseInt(dayDeltaParam);
                    int minuteDelta = Integer.parseInt(minuteDeltaParam);
                    new ScheduleItemResizedEvent(component, eventIdParam, dayDelta, minuteDelta).queue();
                } else if (ITEM_SELECTED_EVENT.equals(eventTypeParam)) {
                    new ScheduleItemSelectedEvent(component, eventIdParam).queue();
                } else if (VIEW_CHANGED_EVENT.equals(eventTypeParam)) {
                    new ScheduleViewChangedEvent(component, viewParam).queue();
                }
            } catch (NumberFormatException ex) {
                // TODO: handle exception (log it probably)
            }
        }
    }

    public void writeInitFunction(FacesContext context, UISchedule component) throws IOException {
        ResponseWriter writer = context.getResponseWriter();
        String clientId = component.getClientId(context);
        writer.writeText(new JSObject("RichFaces.FullCalendar", clientId,
                getOptions(component),
                LOAD_ITEMS_EVENT,
                ITEM_SELECTED_EVENT,
                ITEM_MOVED_EVENT,
                ITEM_RESIZED_EVENT,
                VIEW_CHANGED_EVENT,
                createSubmitEventFunction(context, component)).toScript(),
                null);
    }

    public Map<String, Object> getOptions(UISchedule schedule) throws IOException {
        Map<String, Object> options = new HashMap<String, Object>();
        addOptionIfSet("defaultView", "view", options, schedule);
// TODO shall we numerate firstDayOfWeek as in Calendar (sunday=1,monday=2,etc.) or like in widget(sunday=0,monday=1,etc.)?
        if (schedule.getFirstDay() != null) {
            options.put("firstDay", schedule.getFirstDay() - 1);
        }
        addOptionIfSet("isRTL", "isRTL", options, schedule);
        addOptionIfSet("weekends", "showWeekends", options, schedule);
        addOptionIfSet("weekMode", "weekMode", options, schedule);
        addOptionIfSet("height", "height", options, schedule);
        addOptionIfSet("contentHeight", "contentHeight", options, schedule);
        addOptionIfSet("aspectRatio", "aspectRatio", options, schedule);
        addOptionIfSet("allDaySlot", "allDaySlot", options, schedule);
        addOptionIfSet("allDayText", "allDayText", options, schedule);
        addOptionIfSet("axisFormat", "axisFormat", options, schedule);
        addOptionIfSet("slotMinutes", "slotMinutes", options, schedule);
        addOptionIfSet("defaultEventMinutes", "defaultEventMinutes", options, schedule);
        addOptionIfSet("firstHour", "firstHour", options, schedule);
        addOptionIfSet("minTime", "minTime", options, schedule);
        addOptionIfSet("maxTime", "maxTime", options, schedule);
        addOptionIfSet("editable", "editable", options, schedule);
        addOptionIfSet("disableDragging", "disableDragging", options, schedule);
        addOptionIfSet("disableResizing", "disableResizing", options, schedule);
        addOptionIfSet("dragRevertDuration", "dragRevertDuration", options, schedule);
        addOptionIfSet("dragOpacity", "dragOpacity", options, schedule);
        Map<String, Object> headerOptions = new HashMap<String, Object>(3);
        addOptionIfSet("left", "headerLeft", headerOptions, schedule);
        addOptionIfSet("center", "headerCenter", headerOptions, schedule);
        addOptionIfSet("right", "headerRight", headerOptions, schedule);
        addOptionIfSet("allDayDefault", "allDayByDefault", headerOptions, schedule);
        if (schedule.getDate() != null) {
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(schedule.getDate());
            options.put("year", calendar.get(Calendar.YEAR));
            options.put("month", calendar.get(Calendar.MONTH));
            options.put("date", calendar.get(Calendar.DATE));
        }
        if (headerOptions.size() > 0) {
            options.put("header", headerOptions);
        }
        if (!isClientMode(schedule)) {
            Map<String, Object> initialEvents = new HashMap<String, Object>();
            Date startDate = UISchedule.getFirstDisplayedDay2(schedule);
            Date endDate = UISchedule.getLastDisplayedDate(schedule);
            initialEvents.put("events", schedule.getCalendarData(startDate, endDate));
            Map<String, Object> date = new HashMap<String, Object>();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            date.put("year", calendar.get(Calendar.YEAR));
            date.put("month", calendar.get(Calendar.MONTH));
            date.put("date", calendar.get(Calendar.DATE));
            initialEvents.put("startDate", date);
            date = new HashMap<String, Object>();
            calendar.setTime(endDate);
            date.put("year", calendar.get(Calendar.YEAR));
            date.put("month", calendar.get(Calendar.MONTH));
            date.put("date", calendar.get(Calendar.DATE));
            initialEvents.put("endDate", date);
            options.put("initialEvents", initialEvents);
        } else {
            options.put("events", schedule.getCalendarData(null, null));
        }
        return options;
    }

    protected void addOptionIfSet(String string, String string0, Map<String, Object> options, UISchedule schedule) {
        Object value = schedule.getAttributes().get(string0);
        if (value != null) {
            options.put(string, value);
        }
    }

    public String getFieldId(FacesContext context, UISchedule component, String attribute) {
        return getUtils().clientId(context, component) + NamingContainer.SEPARATOR_CHAR + attribute;
    }

    private boolean isAjaxMode(UISchedule component) {
        String mode = component.getMode();
        return AJAX_MODE.equals(mode) || "".equals(mode) || null == mode;
    }

    private boolean isClientMode(UISchedule component) {
        String mode = component.getMode();
        return CLIENT_MODE.equals(mode);
    }
}
