/**
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

package org.richfaces.component;

import org.ajax4jsf.tests.AbstractAjax4JsfTestCase;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

/**
 * Unit test for simple Component.
 */
public class JSFComponentTest
        extends AbstractAjax4JsfTestCase {
    /**
     * Create the test case
     *
     * @param testName name of the test case
     */
    public JSFComponentTest(String testName) {
        super(testName);
    }
//TODO add test data for week and day
    public void testFirstDisplayedDayCalculation() throws IOException {
        Calendar calendar = Calendar.getInstance();
        UISchedule schedule = (UISchedule) application.createComponent(UISchedule.COMPONENT_TYPE);
        Calendar expectedFirstDisplayedDay;

        final String fileName = System.getProperty("firstDisplayedDayCalculationTestData");
        System.out.println("filename:" + fileName);
        BufferedReader reader = new BufferedReader(new FileReader(fileName));
        String line;
        int lineNr = 0;
        int firstDay;
        boolean showWeekends;
        StringTokenizer tokenizer;
        try {
            while ((line = reader.readLine()) != null) {
                lineNr++;
                System.out.print("line " + lineNr + ": " + line);
                try {
                    tokenizer = new StringTokenizer(line, ",");
                    firstDay = Integer.parseInt(tokenizer.nextToken());
                    showWeekends = Integer.parseInt(tokenizer.nextToken()) == 1;
                    calendar.set(Calendar.YEAR, Integer.parseInt(tokenizer.nextToken()));
                    calendar.set(Calendar.MONTH, Integer.parseInt(tokenizer.nextToken()));
                    calendar.set(Calendar.DATE, Integer.parseInt(tokenizer.nextToken()));
                    Date initialDate = calendar.getTime();
                    calendar.set(Calendar.YEAR, Integer.parseInt(tokenizer.nextToken()));
                    calendar.set(Calendar.MONTH, Integer.parseInt(tokenizer.nextToken()));
                    calendar.set(Calendar.DATE, Integer.parseInt(tokenizer.nextToken()));
                    expectedFirstDisplayedDay = (Calendar) calendar.clone();
                    schedule.setShowWeekends(showWeekends);
                    schedule.setFirstDay(firstDay);
                    schedule.setDate(initialDate);
                    schedule.setView(UISchedule.VIEW_MONTH);
                    calendar.setTime(UISchedule.getFirstDisplayedDay2(schedule));
                    System.out.print(" result: "+calendar.get(Calendar.YEAR)+","+calendar.get(Calendar.MONTH)+","+calendar.get(Calendar.DATE));
                    assert expectedFirstDisplayedDay.get(Calendar.YEAR) == calendar.get(Calendar.YEAR);
                    assert expectedFirstDisplayedDay.get(Calendar.MONTH) == calendar.get(Calendar.MONTH);
                    assert expectedFirstDisplayedDay.get(Calendar.DATE) == calendar.get(Calendar.DATE);
                } catch (Exception e) {
                    throw new RuntimeException("Exception during processing of line " + lineNr + " of test data", e);
                } finally {
                    System.out.println("");
                }
            }
        } finally {
            reader.close();

        }
    }
}